@@ -1,6 +1,5 @@
 from __future__ import annotations
 
-from inspect import cleandoc
 from json import JSONDecodeError
 from types import TracebackType
 from typing import Callable, List, Optional, Type
@@ -14,28 +13,33 @@ from firebolt.common.exception import (
     FireboltEngineError,
     InterfaceError,
 )
-from firebolt.common.urls import ACCOUNT_ENGINE_URL, ENGINE_BY_NAME_URL
+from firebolt.common.urls import ACCOUNT_ENGINE_BY_NAME_URL, ACCOUNT_ENGINE_URL
 from firebolt.common.util import fix_url_schema
 
 DEFAULT_TIMEOUT_SECONDS: int = 5
 
 
 async def _resolve_engine_url(
-    engine_name: str, username: str, password: str, api_endpoint: str
+    engine_name: str,
+    username: str,
+    password: str,
+    api_endpoint: str,
+    account_name: Optional[str] = None,
 ) -> str:
     async with AsyncClient(
         auth=(username, password),
         base_url=api_endpoint,
+        account_name=account_name,
         api_endpoint=api_endpoint,
     ) as client:
         try:
+            account_id = await client.account_id
             response = await client.get(
-                url=ENGINE_BY_NAME_URL,
+                url=ACCOUNT_ENGINE_BY_NAME_URL.format(account_id=account_id),
                 params={"engine_name": engine_name},
             )
             response.raise_for_status()
             engine_id = response.json()["engine_id"]["engine_id"]
-            account_id = await client.account_id
             response = await client.get(
                 url=ACCOUNT_ENGINE_URL.format(
                     account_id=account_id, engine_id=engine_id
@@ -44,15 +48,15 @@ async def _resolve_engine_url(
             response.raise_for_status()
             return response.json()["engine"]["endpoint"]
         except HTTPStatusError as e:
-            # Engine error would be 404
+            # Engine error would be 404.
             if e.response.status_code != 404:
-                raise InterfaceError(f"unable to retrieve engine endpoint: {e}")
+                raise InterfaceError(f"Unable to retrieve engine endpoint: {e}.")
             # Once this is point is reached we've already authenticated with
             # the backend so it's safe to assume the cause of the error is
-            # missing engine
-            raise FireboltEngineError(f"Firebolt engine {engine_name} does not exist")
+            # missing engine.
+            raise FireboltEngineError(f"Firebolt engine {engine_name} does not exist.")
         except (JSONDecodeError, RequestError, RuntimeError, HTTPStatusError) as e:
-            raise InterfaceError(f"unable to retrieve engine endpoint: {e}")
+            raise InterfaceError(f"Unable to retrieve engine endpoint: {e}.")
 
 
 def async_connect_factory(connection_class: Type) -> Callable:
@@ -62,37 +66,41 @@ def async_connect_factory(connection_class: Type) -> Callable:
         password: str = None,
         engine_name: Optional[str] = None,
         engine_url: Optional[str] = None,
+        account_name: Optional[str] = None,
         api_endpoint: str = DEFAULT_API_URL,
     ) -> Connection:
-        cleandoc(
-            """
-            Connect to Firebolt database.
+        """
+        Connect to Firebolt database.
 
-            Connection parameters:
-            database - name of the database to connect
-            username - user name to use for authentication
-            password - password to use for authentication
-            engine_name - name of the engine to connect to
-            engine_url - engine endpoint to use
-            note: either engine_name or engine_url should be provided, but not both
-            """
-        )
+        Args:
+            database: name of the database to connect
+            username: user name to use for authentication
+            password: password to use for authentication
+            engine_name: Optional The name of the engine to connect to
+            engine_url: Optional. The engine endpoint to use
+            account_name: For customers with multiple accounts; if None uses default.
+            api_endpoint(optional): Firebolt API endpoint. Used for authentication.
+
+        Note:
+            Either `engine_name` or `engine_url` should be provided, but not both.
+
+        """
 
         if engine_name and engine_url:
             raise InterfaceError(
-                "Both engine_name and engine_url are provided."
+                "Both engine_name and engine_url are provided. "
                 "Provide only one to connect."
             )
         if not engine_name and not engine_url:
             raise InterfaceError(
-                "Neither engine_name nor engine_url are provided."
+                "Neither engine_name nor engine_url is provided. "
                 "Provide one to connect."
             )
 
         api_endpoint = fix_url_schema(api_endpoint)
-        # This parameters are optional in function signature,
+        # These parameters are optional in function signature
         # but are required to connect.
-        # It's recommended to make them kwargs by PEP 249
+        # PEP 249 recommends making them kwargs.
         for param, name in (
             (database, "database"),
             (username, "username"),
@@ -108,10 +116,11 @@ def async_connect_factory(connection_class: Type) -> Callable:
 
         if engine_name:
             engine_url = await _resolve_engine_url(
-                engine_name,
-                username,
-                password,
-                api_endpoint,
+                engine_name=engine_name,
+                username=username,
+                password=password,
+                account_name=account_name,
+                api_endpoint=api_endpoint,
             )
 
         assert engine_url is not None
@@ -155,7 +164,10 @@ class BaseConnection:
         self._is_closed = False
 
     def cursor(self) -> BaseCursor:
-        """Create new cursor object."""
+        """
+        Create new cursor object.
+        """
+
         if self.closed:
             raise ConnectionClosedError("Unable to create cursor: connection closed")
 
@@ -193,30 +205,30 @@ class BaseConnection:
 
     def commit(self) -> None:
         """Does nothing since Firebolt doesn't have transactions"""
+
         if self.closed:
             raise ConnectionClosedError("Unable to commit: connection closed")
 
 
 class Connection(BaseConnection):
-    cleandoc(
-        """
-        Firebolt asyncronous database connection class. Implements PEP-249.
+    """
+    Firebolt asyncronous database connection class. Implements `PEP 249`_.
 
-        Parameters:
-            engine_url - Firebolt database engine REST API url
-            database - Firebolt database name
-            username - Firebolt account username
-            password - Firebolt account password
-            api_endpoint(optional) - Firebolt API endpoint. Used for authentication
+    Args:
+        engine_url: Firebolt database engine REST API url
+        database: Firebolt database name
+        username: Firebolt account username
+        password: Firebolt account password
+        api_endpoint: Optional. Firebolt API endpoint. Used for authentication.
 
-        Methods:
-            cursor - create new Cursor object
-            close - close the Connection and all it's cursors
+    Note:
+        Firebolt currenly doesn't support transactions
+        so commit and rollback methods are not implemented.
 
-        Firebolt currenly doesn't support transactions so commit and rollback methods
-        are not implemented.
-        """
-    )
+    .. _PEP 249:
+        https://www.python.org/dev/peps/pep-0249/
+
+    """
 
     cursor_class = Cursor
 
