@@ -3,11 +3,9 @@ from __future__ import annotations
 import logging
 import re
 import time
-from datetime import date, datetime
 from enum import Enum
 from functools import wraps
 from inspect import cleandoc
-from json import JSONDecodeError
 from types import TracebackType
 from typing import (
     TYPE_CHECKING,
@@ -18,7 +16,6 @@ from typing import (
     Optional,
     Sequence,
     Tuple,
-    Union,
 )
 
 from aiorwlock import RWLock
@@ -27,9 +24,11 @@ from httpx import Response, codes
 from firebolt.async_db._types import (
     ColType,
     Column,
+    ParameterType,
     RawColType,
     parse_type,
     parse_value,
+    split_format_sql,
 )
 from firebolt.async_db.util import is_db_available, is_engine_running
 from firebolt.client import AsyncClient
@@ -48,13 +47,13 @@ if TYPE_CHECKING:
 
 logger = logging.getLogger(__name__)
 
-ParameterType = Union[int, float, str, datetime, date, bool, Sequence]
 
 JSON_OUTPUT_FORMAT = "JSONCompact"
 
 
 class CursorState(Enum):
     NONE = 1
+    ERROR = 2
     DONE = 3
     CLOSED = 4
 
@@ -99,6 +98,8 @@ class BaseCursor:
         "_rows",
         "_idx",
         "_idx_lock",
+        "_row_sets",
+        "_next_set_idx",
     )
 
     default_arraysize = 1
@@ -107,8 +108,15 @@ class BaseCursor:
         self.connection = connection
         self._client = client
         self._arraysize = self.default_arraysize
+        # These fields initialized here for type annotations purpose
         self._rows: Optional[List[List[RawColType]]] = None
         self._descriptions: Optional[List[Column]] = None
+        self._row_sets: List[
+            Tuple[int, Optional[List[Column]], Optional[List[List[RawColType]]]]
+        ] = []
+        self._rowcount = -1
+        self._idx = 0
+        self._next_set_idx = 0
         self._reset()
 
     def __del__(self) -> None:
@@ -117,19 +125,18 @@ class BaseCursor:
     @property  # type: ignore
     @check_not_closed
     def description(self) -> Optional[List[Column]]:
-        cleandoc(
-            """
-            Provides information about a single result row of a query
-            Attributes:
-            - name
-            - type_code
-            - display_size
-            - internal_size
-            - precision
-            - scale
-            - null_ok
-            """
-        )
+        """
+        Provides information about a single result row of a query
+
+        Attributes:
+            * ``name``
+            * ``type_code``
+            * ``display_size``
+            * ``internal_size``
+            * ``precision``
+            * ``scale``
+            * ``null_ok``
+        """
         return self._descriptions
 
     @property  # type: ignore
@@ -155,31 +162,68 @@ class BaseCursor:
     @property
     def closed(self) -> bool:
         """True if connection is closed, False otherwise."""
+
         return self._state == CursorState.CLOSED
 
     def close(self) -> None:
         """Terminate an ongoing query (if any) and mark connection as closed."""
+
         self._state = CursorState.CLOSED
         # remove typecheck skip  after connection is implemented
         self.connection._remove_cursor(self)  # type: ignore
 
-    def _store_query_data(self, response: Response) -> None:
+    def _append_query_data(self, response: Response) -> None:
         """Store information about executed query from httpx response."""
-        # Empty response is returned for insert query
-        if response.headers.get("content-length", "") == "0":
-            return
-        try:
-            query_data = response.json()
-            self._rowcount = int(query_data["rows"])
-            self._descriptions = [
-                Column(d["name"], parse_type(d["type"]), None, None, None, None, None)
-                for d in query_data["meta"]
-            ]
 
-            # Parse data during fetch
-            self._rows = query_data["data"]
-        except (KeyError, JSONDecodeError) as err:
-            raise DataError(f"Invalid query data format: {str(err)}")
+        row_set: Tuple[
+            int, Optional[List[Column]], Optional[List[List[RawColType]]]
+        ] = (-1, None, None)
+
+        # Empty response is returned for insert query
+        if response.headers.get("content-length", "") != "0":
+            try:
+                query_data = response.json()
+                rowcount = int(query_data["rows"])
+                descriptions = [
+                    Column(
+                        d["name"], parse_type(d["type"]), None, None, None, None, None
+                    )
+                    for d in query_data["meta"]
+                ]
+
+                # Parse data during fetch
+                rows = query_data["data"]
+                row_set = (rowcount, descriptions, rows)
+            except (KeyError, ValueError) as err:
+                raise DataError(f"Invalid query data format: {str(err)}")
+
+        self._row_sets.append(row_set)
+        if self._next_set_idx == 0:
+            # Populate values for first set
+            self._pop_next_set()
+
+    @check_not_closed
+    @check_query_executed
+    def nextset(self) -> Optional[bool]:
+        """
+        Skip to the next available set, discarding any remaining rows
+        from the current set.
+        Returns True if operation was successful,
+        None if there are no more sets to retrive
+        """
+        return self._pop_next_set()
+
+    def _pop_next_set(self) -> Optional[bool]:
+        """
+        Same functionality as .nextset, but doesn't check that query has been executed.
+        """
+        if self._next_set_idx >= len(self._row_sets):
+            return None
+        self._rowcount, self._descriptions, self._rows = self._row_sets[
+            self._next_set_idx
+        ]
+        self._next_set_idx += 1
+        return True
 
     async def _raise_if_error(self, resp: Response) -> None:
         """Raise a proper error if any"""
@@ -200,7 +244,7 @@ class BaseCursor:
             if not await is_engine_running(self.connection, self.connection.engine_url):
                 raise EngineNotRunningError(
                     f"Firebolt engine {self.connection.engine_url} "
-                    "needs to be running to run queries against it"
+                    "needs to be running to run queries against it."
                 )
         resp.raise_for_status()
 
@@ -211,26 +255,52 @@ class BaseCursor:
         self._descriptions = None
         self._rowcount = -1
         self._idx = 0
+        self._row_sets = []
+        self._next_set_idx = 0
 
     async def _do_execute_request(
         self,
         query: str,
-        parameters: Optional[Sequence[ParameterType]] = None,
+        parameters: Sequence[Sequence[ParameterType]],
         set_parameters: Optional[Dict] = None,
-    ) -> Response:
-        resp = await self._client.request(
-            url="/",
-            method="POST",
-            params={
-                "database": self.connection.database,
-                "output_format": JSON_OUTPUT_FORMAT,
-                **(set_parameters or dict()),
-            },
-            content=query,
-        )
+    ) -> None:
+        self._reset()
+        try:
 
-        await self._raise_if_error(resp)
-        return resp
+            queries = split_format_sql(query, parameters)
+
+            for query in queries:
+
+                start_time = time.time()
+                # our CREATE EXTERNAL TABLE queries currently require credentials,
+                # so we will skip logging those queries.
+                # https://docs.firebolt.io/sql-reference/commands/ddl-commands#create-external-table
+                if not re.search("aws_key_id|credentials", query, flags=re.IGNORECASE):
+                    logger.debug(f"Running query: {query}")
+
+                resp = await self._client.request(
+                    url="/",
+                    method="POST",
+                    params={
+                        "database": self.connection.database,
+                        "output_format": JSON_OUTPUT_FORMAT,
+                        **(set_parameters or dict()),
+                    },
+                    content=query,
+                )
+
+                await self._raise_if_error(resp)
+                self._append_query_data(resp)
+                logger.info(
+                    f"Query fetched {self.rowcount} rows in"
+                    f" {time.time() - start_time} seconds"
+                )
+
+            self._state = CursorState.DONE
+
+        except Exception:
+            self._state = CursorState.ERROR
+            raise
 
     @check_not_closed
     async def execute(
@@ -240,40 +310,20 @@ class BaseCursor:
         set_parameters: Optional[Dict] = None,
     ) -> int:
         """Prepare and execute a database query. Return row count."""
-        start_time = time.time()
 
-        # our CREATE EXTERNAL TABLE queries currently require credentials,
-        # so we will skip logging those queries.
-        # https://docs.firebolt.io/sql-reference/commands/ddl-commands#create-external-table
-        if not re.search("aws_key_id|credentials", query, flags=re.IGNORECASE):
-            logger.debug(f"Running query: {query}")
-
-        self._reset()
-        resp = await self._do_execute_request(query, parameters, set_parameters)
-        self._store_query_data(resp)
-        self._state = CursorState.DONE
-        logger.info(
-            f"Query fetched {self.rowcount} rows in {time.time() - start_time} seconds"
-        )
+        params_list = [parameters] if parameters else []
+        await self._do_execute_request(query, params_list, set_parameters)
         return self.rowcount
 
     @check_not_closed
     async def executemany(
         self, query: str, parameters_seq: Sequence[Sequence[ParameterType]]
     ) -> int:
-        cleandoc(
-            """
-            Prepare and execute a database query against all parameter
-            sequences provided. Return last query row count.
-            """
-        )
-        self._reset()
-        resp = None
-        for parameters in parameters_seq:
-            resp = await self._do_execute_request(query, parameters)
-            if resp is not None:
-                self._store_query_data(resp)
-                self._state = CursorState.DONE
+        """
+        Prepare and execute a database query against all parameter
+        sequences provided. Return last query row count.
+        """
+        await self._do_execute_request(query, parameters_seq)
         return self.rowcount
 
     def _parse_row(self, row: List[RawColType]) -> List[ColType]:
@@ -314,12 +364,10 @@ class BaseCursor:
     @check_not_closed
     @check_query_executed
     def fetchmany(self, size: Optional[int] = None) -> List[List[ColType]]:
-        cleandoc(
-            """
-            Fetch the next set of rows of a query result,
-            cursor.arraysize is default size.
-            """
-        )
+        """
+        Fetch the next set of rows of a query result,
+        cursor.arraysize is default size.
+        """
         size = size if size is not None else self.arraysize
         left, right = self._get_next_range(size)
         assert self._rows is not None
@@ -355,32 +403,19 @@ class BaseCursor:
 
 
 class Cursor(BaseCursor):
-    cleandoc(
-        """
-        Class, responsible for executing asyncio queries to Firebolt Database.
-        Should not be created directly, use connection.cursor()
+    """
+    Class, responsible for executing asyncio queries to Firebolt Database.
+    Should not be created directly,
+    use :py:func:`connection.cursor <firebolt.async_db.connection.Connection>`
 
-        Properties:
-        - description - information about a single result row
-        - rowcount - the number of rows produced by last query
-        - closed - True if connection is closed, False otherwise
-        - arraysize - Read/Write, specifies the number of rows to fetch at a time
-        with .fetchmany method
+    Args:
+        description: information about a single result row
+        rowcount: the number of rows produced by last query
+        closed: True if connection is closed, False otherwise
+        arraysize: Read/Write, specifies the number of rows to fetch at a time
+            with the :py:func:`fetchmany` method
 
-        Methods:
-        - close - terminate an ongoing query (if any) and mark connection as closed
-        - execute - prepare and execute a database query
-        - executemany - prepare and execute a database query against all parameter
-          sequences provided
-        - fetchone - fetch the next row of a query result set
-        - fetchmany - fetch the next set of rows of a query result,
-          size is cursor.arraysize by default
-        - fetchall - fetch all remaining rows of a query result
-        - setinputsizes - predefine memory areas for query parameters (does nothing)
-        - setoutputsize - set a column buffer size for fetches of large columns
-          (does nothing)
-        """
-    )
+    """
 
     __slots__ = BaseCursor.__slots__ + ("_async_query_lock",)
 
@@ -397,6 +432,7 @@ class Cursor(BaseCursor):
     ) -> int:
         async with self._async_query_lock.writer:
             return await super().execute(query, parameters, set_parameters)
+        """Prepare and execute a database query"""
 
     @wraps(BaseCursor.executemany)
     async def executemany(
@@ -404,21 +440,29 @@ class Cursor(BaseCursor):
     ) -> int:
         async with self._async_query_lock.writer:
             return await super().executemany(query, parameters_seq)
+        """
+            Prepare and execute a database query against all parameter
+            sequences provided
+        """
 
     @wraps(BaseCursor.fetchone)
     async def fetchone(self) -> Optional[List[ColType]]:
         async with self._async_query_lock.reader:
             return super().fetchone()
+        """Fetch the next row of a query result set"""
 
     @wraps(BaseCursor.fetchmany)
     async def fetchmany(self, size: Optional[int] = None) -> List[List[ColType]]:
         async with self._async_query_lock.reader:
             return super().fetchmany(size)
+        """fetch the next set of rows of a query result,
+          size is cursor.arraysize by default"""
 
     @wraps(BaseCursor.fetchall)
     async def fetchall(self) -> List[List[ColType]]:
         async with self._async_query_lock.reader:
             return super().fetchall()
+        """Fetch all remaining rows of a query result"""
 
     # Iteration support
     @check_not_closed
