@@ -42,7 +42,9 @@ class EngineSettings(FireboltBaseModel):
     """
     Engine Settings.
 
-    See Also: engine_revision.Specification which also contains engine configuration.
+    See Also: :py:class:`EngineRevisionSpecification
+    <firebolt.model.engine_revision.EngineRevisionSpecification>`
+    which also contains engine configuration.
     """
 
     preset: str
@@ -90,7 +92,10 @@ class Engine(FireboltBaseModel):
     """
     A Firebolt engine. Responsible for performing work (queries, data ingestion).
 
-    Engines are configured in Settings and in EngineRevisions.
+    Engines are configured in :py:class:`Settings
+    <firebolt.model.engine.EngineSettings>`
+    and in :py:class:`EngineRevisionSpecification
+    <firebolt.model.engine_revision.EngineRevisionSpecification>`.
     """
 
     # internal
@@ -249,11 +254,10 @@ class Engine(FireboltBaseModel):
         )
 
         # wait for engine to start
-        while (
-            wait_for_startup
-            and engine.current_status_summary
-            != EngineStatusSummary.ENGINE_STATUS_SUMMARY_RUNNING
-        ):
+        while wait_for_startup and engine.current_status_summary not in {
+            EngineStatusSummary.ENGINE_STATUS_SUMMARY_RUNNING,
+            EngineStatusSummary.ENGINE_STATUS_SUMMARY_FAILED,
+        }:
             wait(
                 seconds=5,
                 error_message=f"Could not start engine within {wait_timeout_seconds} seconds.",  # noqa: E501
